<?php

namespace App\Http\Controllers\Api;

use App\Http\Controllers\Controller;
use App\Http\Requests\ProductRequest;
use App\Http\Resources\ProductsResource;
use Illuminate\Support\Facades\DB;
use Illuminate\Http\Request;
use App\Models\Product;
use App\Models\Order;
use App\models\OrderItem;
use Image;
use File;

class ProductsController extends Controller
{
    /**
     * Get product items
     *
     * @return  JSON  data
     */
    public function index()
    {
        $products = Product::with('category')->paginate(100);

        return ProductsResource::collection($products);
    }

    /**
     * Get product items by series of categories
     *
     * @return  JSON  data
     */
    public function seriesByCategory(Request $request)
    {
        $categories = !empty($request) ? json_decode($request->categories) : null;

        if (empty($categories)) {
            return response()->json([
                'success' => false,
                'data' => null,
                'message' => 'Empty category array.'
            ], 422);
        }

        $products = Product::with('category')->whereIn('category_id', $categories)->get();

        // return ProductsResource::collection($products);
        return response()->json([
            'success' => true,
            'data' => $products,
            'message' => 'Products by categories fetch successfully.'
        ], 200);
    }

    /**
     * Record prduct item
     *
     * @param   ProductRequest  $request  request data
     *
     * @return  JSON                      data with success status, data or message
     */
    public function store(ProductRequest $request)
    {
        $product = new Product();

        if (!empty($request->item)) {
            $request->item = json_decode($request->item);
        } else {
            return response()->json([
                'message' => 'Item data enpty.'
            ], 422);
        }

        try {
            DB::beginTransaction();
            $product->is_in_box = 0;
          if(isset($request->item->side_menu_type) && $request->item->side_menu_type == "2"){
            $product->is_in_box = 1;
          }
          if(isset($request->item->items_inside) && !empty($request->item->items_inside)){
            $product->items_inside = json_encode(explode("," , $request->item->items_inside));
          }
            $product->name = $request->item->item_name;
            $product->category_id = $request->item->item_category;
            $product->ingredients = $request->item->item_ingredients;
            $product->allergies = $request->item->item_allergies;
            $product->calorie = $request->item->item_calorie;
            $product->description = $request->item->item_description;
            $product->is_customizable = $request->item->item_customizable;
            $product->price = $request->item->item_price;
            $product->stock = $request->item->item_stock;
            $product->pieces = $request->item->item_pieces;
            $product->limitqty = $request->item->item_limitqty;
            $product->save();

            if ($request->hasFile('item_desktop_pic')) {
                $desktopPicFile = $request->file('item_desktop_pic');
                $desktopPicFileName = $desktopPicFile->getClientOriginalName();
                $desktopPicFileExt  =   pathinfo($desktopPicFileName, PATHINFO_EXTENSION);

                $path = 'img/products/';

                if (!File::exists($path)) {
                    File::makeDirectory($path);
                }

                @unlink($path . $product->id . '.' . $desktopPicFileExt);
                $itemDesktopPic = $product->id . '.' . $desktopPicFileExt;
                Image::make($request->file('item_desktop_pic')->getRealPath())->save($path . $itemDesktopPic);
            }

            if ($request->hasFile('item_mobile_pic')) {
                $mobilePicFile = $request->file('item_mobile_pic');
                $mobilePicFileName = $mobilePicFile->getClientOriginalName();
                $mobilePicFileExt  =   pathinfo($mobilePicFileName, PATHINFO_EXTENSION);

                $path = 'img/products/';

                if (!File::exists($path)) {
                    File::makeDirectory($path);
                }

                @unlink($path . $product->id . '_mobile.' . $mobilePicFileExt);
                $itemMobilePic = $product->id . '_mobile.' . $mobilePicFileExt;
                Image::make($request->file('item_mobile_pic')->getRealPath())->save($path . $itemMobilePic);
            }

            if ($request->hasFile('item_header_pic')) {
                $headerPicFile = $request->file('item_header_pic');
                $headerPicFileName = $headerPicFile->getClientOriginalName();
                $headerPicFileExt  =   pathinfo($headerPicFileName, PATHINFO_EXTENSION);

                $path = 'img/products/';

                if (!File::exists($path)) {
                    File::makeDirectory($path);
                }

                @unlink($path . $product->id . '_item_header.' . $headerPicFileExt);
                $itemHeaderPic = $product->id . '_item_header.' . $headerPicFileExt;
                Image::make($request->file('item_header_pic')->getRealPath())->save($path . $itemHeaderPic);
            }

            $updateProduct = Product::findOrFail($product->id);
            $updateProduct->desktop_pic = (!empty($itemDesktopPic)) ? $itemDesktopPic : null;
            $updateProduct->mobile_pic = (!empty($itemMobilePic)) ? $itemMobilePic : null;
            $updateProduct->header_pic = (!empty($itemHeaderPic)) ? $itemHeaderPic : null;
            $updateProduct->save();

            if ($updateProduct->save()) {
                DB::commit();

                return response()->json([
                    'success' => true,
                    'data1' => $product,
                    'data2' => $updateProduct,
                ], 200);
            } else {
                DB::rollback();

                return response()->json([
                    'success' => false,
                    'message' => 'Error saving product.'
                ], 500);
            }
        } catch (QueryException $qe) {
            DB::rollback();

            if ($request->hasFile('item_desktop_pic')) {
                $file = public_path() . '/img/products/' . $product->desktop_pic;
                \File::delete($file);
            }

            if ($request->hasFile('item_mobile_pic')) {
                $file = public_path() . '/img/products/' . $product->mobile_pic;
                \File::delete($file);
            }

            return response()->json([
                'success' => false,
                'message' => $qe->getMessage()
            ], 500);
        } catch (Exception $e) {
            DB::rollback();

            if ($request->hasFile('item_desktop_pic')) {
                $file = public_path() . '/img/products/' . $product->desktop_pic;
                \File::delete($file);
            }

            if ($request->hasFile('item_mobile_pic')) {
                $file = public_path() . '/img/products/' . $product->mobile_pic;
                \File::delete($file);
            }

            return response()->json([
                'success' => false,
                'message' => $e->getMessage()
            ], 500);
        }
    }

    /**
     * Get product item details
     *
     * @param   int  $id  product item ID
     *
     * @return  Resource  product item details by ID
     */
    public function show($id)
    {
        try {
            $product = Product::with(['category'])->findOrFail($id);

            return response()->json($product, 200);
        } catch (Exception $e) {
            return response()->json([
                'message' => 'Product item not found.'
            ], 404);
        }
    }

    /**
     * Update prduct item by ID
     *
     * @param   ProductRequest  $request  request data
     *
     * @return  JSON                      data with success status, data or message
     */
    public function update(ProductRequest $request, $id)
    {
        $product = Product::findOrFail($id);

        if (!empty($request->item)) {
            $request->item = json_decode($request->item);
        } else {
            return response()->json([
                'message' => 'Item data enpty.'
            ], 422);
        }

        try {
            DB::beginTransaction();
          $product->is_in_box = 0;
          if(isset($request->item->side_menu_type) && $request->item->side_menu_type == "2"){
            $product->is_in_box = 1;
          }
            $product->name = $request->item->item_name;
            $product->category_id = $request->item->item_category;
            $product->ingredients = $request->item->item_ingredients;
            $product->allergies = $request->item->item_allergies;
            $product->calorie = $request->item->item_calorie;
            $product->description = $request->item->item_description;
            $product->is_customizable = $request->item->item_customizable;
            $product->price = $request->item->item_price;
            $product->stock = $request->item->item_stock;
            $product->pieces = $request->item->item_pieces;
            $product->limitqty = $request->item->item_limitqty;
          if(isset($request->item->items_inside) && !empty($request->item->items_inside)){
            $product->items_inside = json_encode(explode("," , $request->item->items_inside));
          }
            $product->save();

            if ($request->hasFile('item_desktop_pic') && !empty($request->file('item_desktop_pic'))) {
                $desktopPicFile = $request->file('item_desktop_pic');
                $desktopPicFileName = $desktopPicFile->getClientOriginalName();
                $desktopPicFileExt  =   pathinfo($desktopPicFileName, PATHINFO_EXTENSION);

                $path = 'img/products/';

                if (!File::exists($path)) {
                    File::makeDirectory($path);
                }

                @unlink($path . $product->id . '.' . $desktopPicFileExt);
                $itemDesktopPic = $product->id . '.' . $desktopPicFileExt;
                Image::make($request->file('item_desktop_pic')->getRealPath())->save($path . $itemDesktopPic);
            }

           if ($request->hasFile('item_mobile_pic') && !empty($request->file('item_mobile_pic'))) {
                $mobilePicFile = $request->file('item_mobile_pic');
                $mobilePicFileName = $mobilePicFile->getClientOriginalName();
                $mobilePicFileExt  =   pathinfo($mobilePicFileName, PATHINFO_EXTENSION);

                $path = 'img/products/';

                if (!File::exists($path)) {
                    File::makeDirectory($path);
                }

                @unlink($path . $product->id . '_mobile.' . $mobilePicFileExt);
                $itemMobilePic = $product->id . '_mobile.' . $mobilePicFileExt;
                Image::make($request->file('item_mobile_pic')->getRealPath())->save($path . $itemMobilePic);
            }
           if ($request->hasFile('item_header_pic') && !empty($request->file('item_header_pic'))) {
                $headerPicFile = $request->file('item_header_pic');
                $headerPicFileName = $headerPicFile->getClientOriginalName();
                $headerPicFileExt  =   pathinfo($headerPicFileName, PATHINFO_EXTENSION);

                $path = 'img/products/';

                if (!File::exists($path)) {
                    File::makeDirectory($path);
                }

                @unlink($path . $product->id . '_item_header.' . $headerPicFileExt);
                $itemHeaderPic = $product->id . '_item_header.' . $headerPicFileExt;
                Image::make($request->file('item_header_pic')->getRealPath())->save($path . $itemHeaderPic);
            }

            $updateProduct = Product::findOrFail($product->id);
            $updateProduct->desktop_pic = (!empty($itemDesktopPic)) ? $itemDesktopPic : $product->desktop_pic;
            $updateProduct->mobile_pic = (!empty($itemMobilePic)) ? $itemMobilePic : $product->mobile_pic;
            $updateProduct->header_pic = (!empty($itemHeaderPic)) ? $itemHeaderPic : $product->header_pic;
            $updateProduct->save();

            if ($updateProduct->save()) {
                DB::commit();

                return response()->json([
                    'success' => true,
                    'data1' => $product,
                    'data2' => $updateProduct,
                ], 200);
            } else {
                DB::rollback();

                return response()->json([
                    'success' => false,
                    'message' => 'Error updating product.'
                ], 500);
            }
        } catch (QueryException $qe) {
            DB::rollback();

            if ($request->hasFile('item_desktop_pic')) {
                $file = public_path() . '/img/products/' . $product->desktop_pic;
                \File::delete($file);
            }

            if ($request->hasFile('item_mobile_pic')) {
                $file = public_path() . '/img/products/' . $product->mobile_pic;
                \File::delete($file);
            }
            if ($request->hasFile('item_header_pic')) {
                $file = public_path() . '/img/products/' . $product->header_pic;
                \File::delete($file);
            }

            return response()->json([
                'success' => false,
                'message' => $qe->getMessage()
            ], 500);
        } catch (Exception $e) {
            DB::rollback();

            if ($request->hasFile('item_desktop_pic')) {
                $file = public_path() . '/img/products/' . $product->desktop_pic;
                \File::delete($file);
            }

            if ($request->hasFile('item_mobile_pic')) {
                $file = public_path() . '/img/products/' . $product->mobile_pic;
                \File::delete($file);
            }
            if ($request->hasFile('item_header_pic')) {
                $file = public_path() . '/img/products/' . $product->header_pic;
                \File::delete($file);
            }

            return response()->json([
                'success' => false,
                'message' => $e->getMessage()
            ], 500);
        }
    }

    /**
     * Destroy product item by ID
     *
     * @param   int  $id    product ID to be deleted
     *
     * @return  JSON        success status with message
     */
    public function destroy($id)
    {
        try {
            DB::beginTransaction();

            $product = Product::findOrFail($id);

            if ($product->delete()) {
                DB::commit();

                $desktopImgFile = public_path() . '/img/products/' . $product->desktop_pic;
                \File::delete($desktopImgFile);

                $mobileImgFile = public_path() . '/img/products/' . $product->mobile_pic;
                \File::delete($mobileImgFile);

                return response()->json([
                    'success' => true,
                    'message' => 'Product item deleted.'
                ], 202);
            }
        } catch (QueryException $qe) {
            DB::rollback();

            return response()->json([
                'success' => false,
                'message' => $qe->getMessage()
            ], 500);
        } catch (\Exception $e) {
            DB::rollback();

            return response()->json([
                'success' => false,
                'message' => 'Product item not found.'
            ], 404);
        }
    }

    /**
     * Get top and worst order product item
     *
     * @return  JSON  product order item stats data
     */
    public function topWorstItemsStats()
    {
        $products = new \stdClass();

        // $productTop = DB::table('order_items')
        // ->select('product_item_id', DB::raw('count(*) as total'))
        // ->join('products', 'order_items.product_item_id', '=', 'products.id')
        // ->groupBy('product_item_id')
        // ->orderBy('product_item_id', 'ASC')
        // ->get();

        $productTop = Product::withCount('orderItems')->orderBy('order_items_count', 'DESC')->limit(5)->get();
        $productWorst = Product::withCount('orderItems')->orderBy('order_items_count', 'ASC')->limit(5)->get();

        $products->top = $productTop;
        $products->worst = $productWorst;

        return response()->json($products, 200);
    }
}
