<?php

namespace App\Http\Controllers\Api;

use App\Http\Controllers\Controller;
use App\Http\Resources\PromosResource;
use App\Http\Requests\PromosRequest;
use Illuminate\Support\Facades\DB;
use Illuminate\Http\Request;
use App\Models\PromoCategory;
use App\Models\PromoItem;
use App\Models\Promo;
use Image;
use File;

class PromosController extends Controller
{
    /**
     * Get all promo items
     *
     * @return  PromosResource  promo items collection data
     */
    public function index()
    {
        // $promos = Promo::paginate(10);
        $promos = Promo::with(['promoCategories' => function($query) {
            $query->with(['productCategories'])->get();
        }, 'promoItems' => function($query1) {
            $query1->with(['productItems'])->get();
        }])->paginate(10);

        return PromosResource::collection($promos);
    }

    /**
     * Get all promo items that are active status
     *
     * @return  PromosResource  promo items active status collection data
     */
    public function current()
    {
        $promos = Promo::with(['promoCategories' => function($query) {
            $query->with(['productCategories'])->get();
        }, 'promoItems' => function($query1) {
            $query1->with(['productItems'])->get();
        }])->where('is_active', '1')->paginate(10);

        return PromosResource::collection($promos);
    }

    /**
     * Record promo
     *
     * @param   PromosRequest  $request  request data
     *
     * @return  JSON                     success status with data or message
     */
    public function store(PromosRequest $request) {
        $request->promo = json_decode($request->promo);
        $promo = new Promo();

        try {
            DB::beginTransaction();

            $promo->name = $request->promo->promo_name;
            $promo->code = $request->promo->promo_code;
            $promo->discount = $request->promo->promo_discount;
            $promo->applies_to = $request->promo->promo_applies_to;
            $promo->start_date = $request->promo->promo_start_date;
            $promo->end_date = $request->promo->promo_end_date;
            $promo->save();

            foreach ($request->promo->promo_categories_options_selected as $key => $promoCategory) {
                $promoCategories = new PromoCategory();
                $promoCategories->category_id = $promoCategory->id;
                $promoCategories->promo_id = $promo->id;
                $promoCategories->save();
            }

            foreach ($request->promo->promo_items_options_selected as $key => $promoItem) {
                $promoItems = new PromoItem();
                $promoItems->item_id = $promoItem->id;
                $promoItems->category_id = $promoItem->category_id;
                $promoItems->promo_id = $promo->id;
                $promoItems->save();
            }

            if ($request->hasFile('promo_desktop_pic')) {
                $desktopPicFile = $request->file('promo_desktop_pic');
                $desktopPicFileName = $desktopPicFile->getClientOriginalName();
                $desktopPicFileExt = pathinfo($desktopPicFileName, PATHINFO_EXTENSION);

                $path = 'img/promos/';

                if (!File::exists($path)) {
                    File::makeDirectory($path);
                }

                @unlink($path. $promo->id . '.' . $desktopPicFileExt);
                $promoDesktopPic = $promo->id . '.' . $desktopPicFileExt;
                Image::make($request->file('promo_desktop_pic')->getRealPath())->save($path. $promoDesktopPic);
            }

            if ($request->hasFile('promo_mobile_pic')) {
                $mobilePicFile = $request->file('promo_mobile_pic');
                $mobilePicFileName = $mobilePicFile->getClientOriginalName();
                $mobilePicFileExt = pathinfo($mobilePicFileName, PATHINFO_EXTENSION);

                $path = 'img/promos/';

                if (!File::exists($path)) {
                    File::makeDirectory($path);
                }

                @unlink($path. $promo->id . '_mobile.' . $mobilePicFileExt);
                $promoMobilePic = $promo->id . '_mobile.' . $mobilePicFileExt;
                Image::make($request->file('promo_mobile_pic')->getRealPath())->save($path. $promoMobilePic);
            }

            $updatePromo = Promo::findOrFail($promo->id);
            $updatePromo->desktop_pic = (!empty($promoDesktopPic)) ? $promoDesktopPic : null;
            $updatePromo->mobile_pic = (!empty($promoMobilePic)) ? $promoMobilePic : null;

            if ($updatePromo->save()) {
                DB::commit();

                return response()->json([
                    'success' => true,
                    'data1' => $promo,
                    'data2' => $updatePromo,
                ], 200);
            } else {
                DB::rollback();

                return response()->json([
                    'success' => false,
                    'message' => 'Error saving promo.'
                ], 500);
            }
        } catch(QueryException $qe) {
            DB::rollback();

            if ($request->hasFile('promo_desktop_pic')) {
                $file = public_path() . '/img/promos/' . $promo->desktop_pic;
                \File::delete($file);
            }

            if ($request->hasFile('promo_mobile_pic')) {
                $file = public_path() . '/img/promos/' . $promo->mobile_pic;
                \File::delete($file);
            }

            return response()->json([
                'success' => false,
                'message' => $qe->getMessage()
            ], 500);
        } catch(\Exception $e) {
            DB::rollback();

            if ($request->hasFile('promo_desktop_pic')) {
                $file = public_path() . '/img/promos/' . $promo->desktop_pic;
                \File::delete($file);
            }

            if ($request->hasFile('promo_mobile_pic')) {
                $file = public_path() . '/img/promos/' . $promo->mobile_pic;
                \File::delete($file);
            }

            return response()->json([
                'success' => false,
                'message' => $e->getMessage()
            ], 500);
        }
    }

    /**
     * Get promo details
     *
     * @param   int  $id  promo ID
     *
     * @return  JSON      promo data
     */
    public function show($id)
    {
        try {
            $promo = Promo::with(['promoCategories' => function($query) use ($id) {
                $query->with(['productCategories'])->get();
            }, 'promoItems' => function($query1) use ($id) {
                $query1->with(['productItems'])->get();
            }])->findOrFail($id);

            return response()->json($promo, 200);
        } catch (Exception $e) {
            return response()->json([
                'message' => 'Promo data not found.'
            ], 404);
        }
    }

    public function update(PromosRequest $request, $id)
    {
        $request->promo = json_decode($request->promo);
        $promoUpdate = Promo::findOrFail($id);

        try {
            DB::beginTransaction();

            $promoUpdate->name = $request->promo->promo_name;
            $promoUpdate->code = $request->promo->promo_code;
            $promoUpdate->discount = $request->promo->promo_discount;
            $promoUpdate->applies_to = $request->promo->promo_applies_to;
            $promoUpdate->start_date = $request->promo->promo_start_date;
            $promoUpdate->end_date = $request->promo->promo_end_date;
            $promoUpdate->desktop_pic = $promoUpdate->desktop_pic;
            $promoUpdate->mobile_pic = $promoUpdate->mobile_pic;

            PromoCategory::where('promo_id', $id)->delete();
            PromoItem::where('promo_id', $id)->delete();

            foreach ($request->promo->promo_categories_options_selected as $key => $promoCategory) {
                $promoCategories = new PromoCategory();
                $promoCategories->category_id = $promoCategory->id;
                $promoCategories->promo_id = $id;
                $promoCategories->save();
            }

            foreach ($request->promo->promo_items_options_selected as $key => $promoItem) {
                $promoItems = new PromoItem();
                $promoItems->item_id = $promoItem->id;
                $promoItems->category_id = $promoItem->category_id;
                $promoItems->promo_id = $id;
                $promoItems->save();
            }

            if ($request->hasFile('promo_desktop_pic')) {
                $desktopPicFile = $request->file('promo_desktop_pic');
                $desktopPicFileName = $desktopPicFile->getClientOriginalName();
                $desktopPicFileExt = pathinfo($desktopPicFileName, PATHINFO_EXTENSION);

                $path = 'img/promos/';

                if (!File::exists($path)) {
                    File::makeDirectory($path);
                }

                @unlink($path. $id . '.' . $desktopPicFileExt);
                $promoDesktopPic = $id . '.' . $desktopPicFileExt;
                Image::make($request->file('promo_desktop_pic')->getRealPath())->save($path. $promoDesktopPic);
            }

            if ($request->hasFile('promo_mobile_pic')) {
                $mobilePicFile = $request->file('promo_mobile_pic');
                $mobilePicFileName = $mobilePicFile->getClientOriginalName();
                $mobilePicFileExt = pathinfo($mobilePicFileName, PATHINFO_EXTENSION);

                $path = 'img/promos/';

                if (!File::exists($path)) {
                    File::makeDirectory($path);
                }

                @unlink($path. $id . '_mobile.' . $mobilePicFileExt);
                $promoMobilePic = $id . '_mobile.' . $mobilePicFileExt;
                Image::make($request->file('promo_mobile_pic')->getRealPath())->save($path. $promoMobilePic);
            }

            if (!empty($promoDesktopPic) || !empty($promoMobilePic)) {
                $updatePromo = Promo::findOrFail($id);
                $updatePromo->desktop_pic = (!empty($promoDesktopPic)) ? $promoDesktopPic : null;
                $updatePromo->mobile_pic = (!empty($promoMobilePic)) ? $promoMobilePic : null;
                $updatePromo->save();
            }

            if ($promoUpdate->save()) {
                DB::commit();

                return response()->json([
                    'success' => true,
                    'data1' => $promoUpdate,
                    'data2' => !empty($updatePromo) ? $updatePromo : null,
                ], 200);
            } else {
                DB::rollback();

                return response()->json([
                    'success' => false,
                    'message' => 'Error updating promo.'
                ], 500);
            }
        } catch(QueryException $qe) {
            DB::rollback();

            if ($request->hasFile('promo_desktop_pic')) {
                $file = public_path() . '/img/promos/' . $promoUpdate->desktop_pic;
                \File::delete($file);
            }

            if ($request->hasFile('promo_mobile_pic')) {
                $file = public_path() . '/img/promos/' . $promoUpdate->mobile_pic;
                \File::delete($file);
            }

            return response()->json([
                'success' => false,
                'message' => $qe->getMessage()
            ], 500);
        } catch(\Exception $e) {
            DB::rollback();

            if ($request->hasFile('promo_desktop_pic')) {
                $file = public_path() . '/img/promos/' . $promoUpdate->desktop_pic;
                \File::delete($file);
            }

            if ($request->hasFile('promo_mobile_pic')) {
                $file = public_path() . '/img/promos/' . $promoUpdate->mobile_pic;
                \File::delete($file);
            }

            return response()->json([
                'success' => false,
                'message' => $e->getMessage()
            ], 500);
        }
    }

    /**
     * Remove promo and its categoriess and items
     *
     * @param   int     $id     promo ID
     *
     * @return  JSON            status with data or message
     */
    public function destroy($id)
    {
        try {
            DB::beginTransaction();

            $promo = Promo::findOrFail($id);

            if ($promo->delete()) {
                PromoCategory::where('promo_id', $id)->delete();
                PromoItem::where('promo_id', $id)->delete();

                DB::commit();

                $desktopImgFile = public_path() . '/img/promos/' . $promo->desktop_pic;
                \File::delete($desktopImgFile);

                $mobileImgFile = public_path() . '/img/promos/' . $promo->mobile_pic;
                \File::delete($mobileImgFile);

                return response()->json([
                    'success' => true,
                    'message' => 'Promo deleted.'
                ], 202);
            }
        } catch(QueryException $qe) {
            DB::rollback();

            return response()->json([
                'success' => false,
                'message' => $qe->getMessage()
            ], 500);
        } catch(\Exception $e) {
            DB::rollback();

            return response()->json([
                'success' => false,
                'message' => 'Promo not found.'
            ], 404);
        }
    }

    /**
     * trigger promo active status
     *
     * @param   int     $id         promo ID
     * @param   int     $is_active  promo active status
     *
     * @return  JSON                status with data or message
     */
    public function triggerActiveStatus($id, $is_active) {
        try {
            DB::beginTransaction();

            if ($is_active === null) {
                return response()->json([
                    'success' => false,
                    'message' => 'Promo ID active status not found.'
                ], 422);
            }

            $promo = Promo::findOrFail($id);
            $promo->is_active = $is_active;

            if ($promo->save()) {
                DB::commit();

                return response()->json([
                    'success' => true,
                    'data' => $promo
                ], 200);
            } else {
                return response()->json([
                    'success' => false,
                    'message' => 'Error changing promo active status.'
                ], 404);
            }
        } catch(QueryException $qe) {
            DB::rollback();

            return response()->json([
                'success' => false,
                'message' => $qe->getMessage()
            ], 500);
        } catch(\Exception $e) {
            DB::rollback();

            return response()->json([
                'success' => false,
                'message' => 'Promo not found.'
            ], 404);
        }
    }
    public function promoCode(Request $request){
        $Promo = new Promo;
        $promo_discount = $Promo::with('promoItems')->where('promos.code', $request->promoCode)
            ->where('is_active' , '1')->first();
        return $promo_discount;
    }
}
